/*
** Module   : ClientState.java
** Abstract : contains data to synchronize the server with the client's state
**
** Copyright (c) 2008-2009, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -T- --JPRM-- ----------------Description-----------------
** 001 GES 20080613 NEW   @38878 Created initial version to contain a simple
**                               set of client-side state.
** 002 SIY 20081209 CHG   @40856 Added list of IDs of wrapped frames.
** 003 CA  20090413 CHG   @41772 Added tracking of changed VISIBLE/HIDDEN 
**                               widget attributes. When these attributes get
**                               changed on client-side, they are sent to the
**                               server for update.
*/

package com.goldencode.p2j.ui;

import java.io.*;

/**
 * This class is a container for client-side state.
 */
public class ClientState
implements Externalizable
{
   /** List of frame IDs to deregister. */
   public int[] deregister = null;
   
   /** Frame DOWN info. */
   public int[] down = null;
   
   /** IDs of DOWN frames.*/
   public int[] downIDs = null;

   /** IDs of wrapped down frames. */
   public int[] wrapped = null;
   
   /** Flag to indicate if there was a pause during the last operation. */
   public boolean hadPause = false;
   
   /** 
    * List of (widgetId, state) pairs, for which the HIDDEN flag changed. Row
    * on index 0 keeps widget ids, row on index 1 keeps corresponding HIDDEN 
    * flag state.
    */
   public int[][] hiddenWidgets;

   /** 
    * List of (widgetId, state) pairs, for which the VISIBLE flag changed. Row
    * on index 0 keeps widget ids, row on index 1 keeps corresponding VISIBLE 
    * flag state.
    */
   public int[][] visibleWidgets;

   /**
    * The code of last event (LAST-EVENT:FUNCTION)
    * <p>
    * If this the type of this field will be "int" there will be following 
    * conflict: not initialized value will be 0, but at the same time
    * 0 - is a valid code of the key (Ctrl-@). The Integer class is used here
    * to distinguish uninitialized value (null) and 0 (valid function code) 
    */
   public Integer functionKey;
   
   /**
    * Construct an empty instance.
    */
   public ClientState()
   {
   }
   
   /**
    * Replacement for the default object reading method.
    * 
    * @param    in
    *           Input source from which fields will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      deregister     = (int[]) in.readObject();
      down           = (int[]) in.readObject();
      downIDs        = (int[]) in.readObject();
      wrapped        = (int[]) in.readObject();
      hadPause       = in.readBoolean();
      hiddenWidgets  = (int[][]) in.readObject();
      visibleWidgets = (int[][]) in.readObject();
      functionKey    = (Integer) in.readObject();
   }

   /**
    * Replacement for the default object writing method.
    * 
    * @param    out
    *           The output destination to which fields will be saved.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeObject(deregister);
      out.writeObject(down);
      out.writeObject(downIDs);
      out.writeObject(wrapped);
      out.writeBoolean(hadPause);
      out.writeObject(hiddenWidgets);
      out.writeObject(visibleWidgets);
      out.writeObject(functionKey);
   }
}
